import os
from PyQt5 import QtCore, QtGui, QtWidgets, Qt
from PyQt5.QtWidgets import *
from PyQt5.QtSql import QSqlDatabase
import logging
from PyQt5.QtCore import QDir, QFile, Qt
from PyQt5.QtCore import QSize
from PyQt5.QtGui import QIcon
from datetime import datetime
from Components.ChatBotView.cevapVer import Chat, pairs, reflections
from Components.ChatBotView.sqlDialog import SqlKonusmaModeli
from configuration import Configuration
logfilename = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'chat.log')
logging.basicConfig(filename=logfilename, level=logging.DEBUG)
logger = logging.getLogger("logger")

# region Veritabanı bağlantı işlemleri
def veritabaniBaglan():
    veritabani = QSqlDatabase.database()
    if not veritabani.isValid():
        veritabani = QSqlDatabase.addDatabase("QSQLITE")
        if not veritabani.isValid():
            logger.error("Veritabanı Eklenemedi !")

    yaz_dir = QDir()
    if not yaz_dir.mkpath("."):
        logger.error("Yazılabilir dizin oluşturulamadı !")

    # Erişilebilir bir veritabanı dosyası oluşturulmuştur.
    chat_database = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'chat-database.sqlite3')

    # Veritabanı mevcut değilse open() fonksiyonu SQLite'ı oluşturacaktır.
    veritabani.setDatabaseName(chat_database)
    if not veritabani.open():
        logger.error("Veritabanı Açılamadı")
        QFile.remove(dosyaAdi)
# endregion



#region SohbetBalonları

ballon = """
<table border="0" cellpadding="0" cellspacing="0">
      <tr>
         <td><img src=":/icon/images/sol_ust_beyaz.gif"></td>
         <td bgcolor="#ffffff"></td>
         <td><img src=":/icon/images/sag_ust_beyaz.gif"></td>
		 <td width="40"><img src=":/icon/images/transparent.gif"></td>
      </tr>
      <tr>
         <td background=":/icon/images/sol_orta_beyaz.gif">&nbsp;</td>
         <td bgcolor="#ffffff" style="">{}<div align="right" style="font-size: 9px;color: #1BA44C;">{}</div></td>
         <td bgcolor="#ffffff">&nbsp;</td>
		 <td width="40"><img src=":/icon/images/transparent.gif"></td>
      </tr>
      <tr>
         <td><img src=":/icon/images/sol_alt_beyaz.gif"></td>
         <td bgcolor="#ffffff"><img src=":/icon/images/transparent.gif"></td>
         <td><img src=":/icon/images/sag_alt_beyaz.gif"></td>
		 <td width="40"><img src=":/icon/images/transparent.gif"></td>
      </tr>
</table>
"""

ballon_Kullanici = """
<table border="0" cellpadding="0" cellspacing="0" align="right">
      <tr>
         <td width="40"><img src=":/icon/images/transparent.gif"></td>
		 <td><img src=":/icon/images/sol_ust_yesil.gif"></td>
         <td bgcolor="#e9fedd"><img src=":/icon/images/transparent.gif"></td>
         <td><img src=":/icon/images/sag_ust_yesil.gif"></td>
      </tr>
      <tr >
		 <td width="40"><img src=":/icon/images/transparent.gif"></td>
         <td bgcolor="#e9fedd">&nbsp;</td>
         <td bgcolor="#e9fedd" style="">{}<div align="right" style="font-size: 9px;color: #1BA44C;">{}</div></td>
         </td>
         <td background=":/icon/images/sag_orta_yesil.gif">&nbsp;</td>
      </tr>
      <tr>
		 <td width="40"><img src=":/icon/images/transparent.gif"></td>
         <td><img src=":/icon/images/sol_alt_yesil.gif"></td>
         <td bgcolor="#e9fedd"><img src=":/icon/images/transparent.gif"></td>
         <td><img src=":/icon/images/sag_alt_yesil.gif"></td>
      </tr>
</table>
"""

#endregion

class SecondWindow(QtWidgets.QDialog):
    def __init__(self):
        super(SecondWindow, self).__init__()
        self.setUI()

    def setUI(self):
        centerPoint = QDesktopWidget().availableGeometry().center()
        self.setGeometry(centerPoint.x() - 180, centerPoint.y() + 96, 270, 157)
        self.setMinimumSize(QSize(270, 157))
        self.setMaximumSize(QSize(270, 157))
        self.setWindowTitle("Emojiler")
        self.setWindowIcon(QIcon(':/icon/images/smile.png'))
        """
            Emojiye tıklanıldığında fonksiyon, emojinin karşılığını password adında görünmeyen bir label'a yazıyor. Dialog bittiğinde ise
            label'daki yazı userinput'a gönderiliyor. Farklı yollar denedik. Direkt olarak butona tıklandığında ana pencere sınıfına ulaşıp orada
            setText yapmayı denedik ama userinput değiştiği halde ekrana yazdırmıyordu. Userinputun değiştiğini ise konsola yazdırarak kontrol etmiştik.
            Signal kullanmayı da denedik. Emojiye tıklanıldığında direkt kapanmasından dolayı QDialog kullanmaya karar verdik.
        """

        self.password = QtWidgets.QLineEdit(self)
        self.password.setGeometry(200, 300, 50, 30)
        self.gulucuk = QPushButton(self)
        self.gulucuk.setObjectName("gulucuk")
        self.gulucuk.setGeometry(5, 5, 70, 60)
        self.gulucuk.clicked.connect(self.gulucukKoy)

        self.uzgun = QPushButton(self)
        self.uzgun.setObjectName("uzgun")

        self.uzgun.setGeometry(95, 5, 70, 60)
        self.uzgun.clicked.connect(self.uzgunSuratKoy)
        self.gulme = QPushButton(self)
        self.gulme.setObjectName("gulme")

        self.gulme.setGeometry(185, 5, 70, 60)
        self.gulme.clicked.connect(self.gul)

        self.gozKirpma = QPushButton(self)
        self.gozKirpma.setObjectName("gozKirpma")

        self.gozKirpma.setGeometry(5, 80, 70, 60)
        self.gozKirpma.clicked.connect(self.gozKirp)
        self.duzSurat = QPushButton(self)
        self.duzSurat.setObjectName("duzSurat")

        self.duzSurat.setGeometry(95, 80, 70, 60)
        self.duzSurat.clicked.connect(self.duzSuratKoy)
        self.sasirma = QPushButton(self)
        self.sasirma.setObjectName("sasirma")

        self.sasirma.setGeometry(185, 80, 70, 60)
        self.sasirma.clicked.connect(self.sasirmaKoy)

    def gulucukKoy(self):
        self.password.setText(":) ")
        self.accept()

    def gul(self):
        self.password.setText(":D ")
        self.accept()

    def uzgunSuratKoy(self):
        self.password.setText(":( ")
        self.accept()

    def sasirmaKoy(self):
        self.password.setText(":O ")
        self.accept()

    def duzSuratKoy(self):
        self.password.setText(":| ")
        self.accept()

    def gozKirp(self):
        self.password.setText(";) ")
        self.accept()





class UcChatBotView(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.mainWindow = parent
        self.setupUi(self)
        self.pushButton_smile.clicked.connect(self.emojiPage)

        # self.pushButton_smile.clicked.connect(self.emojiPage)
        now = datetime.now()
        zaman = now.strftime("%H:%M")
        karsilama_metni = "Nasıl yardımcı olabilirim?"
        karsilama = ballon.format(karsilama_metni, zaman)
        self.textEdit_message.append(karsilama)
        self.textEdit_message.append("\n")
        self.textEdit_message.setReadOnly(True)
        # self.setWindowTitle("PyNar Kod Editörü")
        # self.setWindowIcon(QIcon(':/icon/images/pynar_image.png'))
        self.lineEdit_sendMessage.returnPressed.connect(self.AddToChatLogUser)
        self.lineEdit_sendMessage.setEnabled(True)
        self.lineEdit_sendMessage.setFocus()

    def emojiPage(self):
        self.SW = SecondWindow()
        if self.SW.exec_():
            self.lineEdit_sendMessage.setText(self.lineEdit_sendMessage.text() + " " + self.SW.password.text())
        self.lineEdit_sendMessage.setFocus()

    def UpdateCycle(self):
        """Retrieves a new bot message and appends to the chat log."""
        bmsg = self.v.getBotMessage()
        self.chatlog.setAlignment(Qt.AlignRight)
        [self.chatlog.append(m) for m in bmsg]
        self.userinput.setFocus()

    def ReplaceToEmoji(self, message):
        newMessage = message
        if ":)" in message:
            newMessage = newMessage.replace(':)', ' &#128522; ')
        if ":(" in message:
            newMessage = newMessage.replace(':(', ' &#128542; ')
        if ":D" in message:
            newMessage = newMessage.replace(':D', ' &#128516; ')
        if ":O" in message:
            newMessage = newMessage.replace(':O', ' &#128558; ')
        if ";)" in message:
            newMessage = newMessage.replace(';)', ' &#128521; ')
        if ":|" in message:
            newMessage = newMessage.replace(':|', ' &#128528; ')
        return newMessage

    def AddToChatLogUser(self):
        """Takes guest's entry and appends to the chatlog"""
        message = self.lineEdit_sendMessage.text()
        if message != '':
            umsg = Chat(message, pairs, reflections)
            umsg.converse(message, quit='tamam')
            SqlKonusmaModeli().mesaj_ekle(message, umsg.__repr__())
            self.textEdit_message.setAlignment(Qt.AlignLeft)
            now = datetime.now()
            zaman = now.strftime("%H:%M")
            ballonmesaj = ballon.format(umsg.__repr__(), zaman)
            splitting = self.ReplaceToEmoji(message)
            ballonKullanici = ballon_Kullanici.format(splitting, zaman)
            self.textEdit_message.append(ballonKullanici)
            self.textEdit_message.append("\n")
            self.textEdit_message.setAlignment(Qt.AlignRight)
            self.textEdit_message.append(ballonmesaj)
            self.textEdit_message.append("\n")
            self.textEdit_message.moveCursor(QtGui.QTextCursor.End)
            self.lineEdit_sendMessage.setText("")


    def setupUi(self, Form):
        Form.setObjectName("Form")
        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setObjectName("gridLayout")
        self.frame_sagMenuParent = QtWidgets.QFrame()
        self.frame_sagMenuParent.setObjectName("FrameSagMenuParent")
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.frame_sagMenuParent.sizePolicy().hasHeightForWidth())
        self.frame_sagMenuParent.setSizePolicy(sizePolicy)
        self.frame_sagMenuParent.setMinimumSize(QtCore.QSize(335, 0))
        self.frame_sagMenuParent.setMaximumSize(QtCore.QSize(335, 16777215))
        self.frame_sagMenuParent.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame_sagMenuParent.setFrameShadow(QtWidgets.QFrame.Raised)
        self.gridLayout.addWidget(self.frame_sagMenuParent, 0, 0, 1, 1)

        self.gridLayout_2 = QtWidgets.QGridLayout(self.frame_sagMenuParent)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.verticalLayout_sagMenu = QtWidgets.QVBoxLayout()
        self.verticalLayout_sagMenu.setObjectName("verticalLayout_sagMenu")
        self.frame_sagMenu2 = QtWidgets.QFrame(self.frame_sagMenuParent)
        self.frame_sagMenu2.setMinimumSize(QtCore.QSize(0, 70))
        self.frame_sagMenu2.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame_sagMenu2.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_sagMenu2.setObjectName("FrameSagMenu2")
        self.label_robot = QtWidgets.QLabel(self.frame_sagMenu2)
        self.label_robot.setGeometry(QtCore.QRect(20, 0, 91, 71))
        self.label_robot.setText("")
        self.label_robot.setObjectName("LabelRobot")
        self.verticalLayout_sagMenu.addWidget(self.frame_sagMenu2)
        self.textEdit_message = QtWidgets.QTextEdit(self.frame_sagMenuParent)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.textEdit_message.sizePolicy().hasHeightForWidth())
        self.textEdit_message.setSizePolicy(sizePolicy)
        self.textEdit_message.setMinimumSize(QtCore.QSize(300, 200))

        self.textEdit_message.setObjectName("TextEditMessage")
        self.textEdit_message.verticalScrollBar().setObjectName("chatbotScrollBarV")
        self.verticalLayout_sagMenu.addWidget(self.textEdit_message)
        self.frame_sagMenu3 = QtWidgets.QFrame(self.frame_sagMenuParent)
        self.frame_sagMenu3.setMinimumSize(QtCore.QSize(0, 75))
        self.frame_sagMenu3.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame_sagMenu3.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_sagMenu3.setObjectName("FrameSagMenu3")
        self.lineEdit_sendMessage = QtWidgets.QLineEdit(self.frame_sagMenu3)
        self.lineEdit_sendMessage.setGeometry(QtCore.QRect(0, 20, 307, 55))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEdit_sendMessage.sizePolicy().hasHeightForWidth())
        self.lineEdit_sendMessage.setSizePolicy(sizePolicy)
        self.lineEdit_sendMessage.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.lineEdit_sendMessage.setCursor(QtGui.QCursor(QtCore.Qt.ArrowCursor))
        self.lineEdit_sendMessage.setEchoMode(QtWidgets.QLineEdit.Normal)
        self.lineEdit_sendMessage.setAlignment(QtCore.Qt.AlignLeading | QtCore.Qt.AlignLeft | QtCore.Qt.AlignVCenter)
        self.lineEdit_sendMessage.setObjectName("LineEditSendMessage")
        self.pushButton_smile = QtWidgets.QPushButton(self.frame_sagMenu3)
        self.pushButton_smile.setGeometry(QtCore.QRect(0, 18, 60, 60))
        self.pushButton_smile.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))
        self.pushButton_smile.setText("")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icon/images/happy.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_smile.setIcon(icon)
        self.pushButton_smile.setIconSize(QtCore.QSize(45, 45))
        self.pushButton_smile.setObjectName("PushButtonSmile")
        self.verticalLayout_sagMenu.addWidget(self.frame_sagMenu3)
        self.gridLayout_2.addLayout(self.verticalLayout_sagMenu, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.frame_sagMenuParent, 0, 0, 1, 1)


    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))




