#!/usr/bin/env python
import os

from PyQt5 import QtCore, QtGui
from PyQt5.QtCore import QSize, Qt
from PyQt5.QtGui import QIcon
from PyQt5.QtWebEngineWidgets import QWebEngineView
from PyQt5.QtWidgets import (QApplication, QDialog, QHBoxLayout, QListView, QListWidget, QListWidgetItem, QPushButton,
                             QStackedWidget, QVBoxLayout, QWidget)
from configuration import Configuration


class TanimSayfasi(QWidget):
    def __init__(self, dosyaAdi):
        super(TanimSayfasi, self).__init__()
        mainLayout = QVBoxLayout()
        view = QWebEngineView()
        self.c = Configuration()#bundan sonra dosya yolunu configuration'dan alalım.
        dosyaPath = self.c.getHomeDir()+self.c.getHtmlHelpPath("html_help_path")
        view.load(QtCore.QUrl.fromLocalFile(dosyaPath + dosyaAdi + '/aciklama.html'))
        mainLayout.addWidget(view)
        mainLayout.addStretch(1)
        self.setLayout(mainLayout)


class KullanimSekli(QWidget):
    def __init__(self, dosyaAdi):
        super(KullanimSekli, self).__init__()
        mainLayout = QVBoxLayout()
        view = QWebEngineView()
        self.c = Configuration()#bundan sonra dosya yolunu configuration'dan alalım.
        dosyaPath = self.c.getHomeDir()+self.c.getHtmlHelpPath("html_help_path")
        view.load(QtCore.QUrl.fromLocalFile(dosyaPath + dosyaAdi + '/kullanim.html'))
        mainLayout.addWidget(view)
        mainLayout.addStretch(1)
        self.setLayout(mainLayout)

class Ornekler(QWidget):
    def __init__(self, dosyaAdi):
        super(Ornekler, self).__init__()
        mainLayout = QVBoxLayout()
        view = QWebEngineView()
        self.c = Configuration()#bundan sonra dosya yolunu configuration'dan alalım.
        dosyaPath = self.c.getHomeDir()+self.c.getHtmlHelpPath("html_help_path")
        view.load(QtCore.QUrl.fromLocalFile(dosyaPath + dosyaAdi + '/ornekler.html'))
        mainLayout.addWidget(view)
        mainLayout.addStretch(1)
        self.setLayout(mainLayout)

class TreeHelpDialog(QDialog):
    def __init__(self, dosyaAdi):
        super(TreeHelpDialog, self).__init__()
        self.contentsWidget = QListWidget()
        self.contentsWidget.setViewMode(QListView.IconMode)
        self.contentsWidget.setIconSize(QSize(96, 84))
        self.contentsWidget.setMovement(QListView.Static)
        self.contentsWidget.setMaximumWidth(128)
        self.contentsWidget.setSpacing(12)
        self.pagesWidget = QStackedWidget()
        self.pagesWidget.addWidget(TanimSayfasi(dosyaAdi))
        self.pagesWidget.addWidget(KullanimSekli(dosyaAdi))
        self.pagesWidget.addWidget(Ornekler(dosyaAdi))
        closeButton = QPushButton("Kapat")
        self.createIcons()
        self.contentsWidget.setCurrentRow(0)
        closeButton.clicked.connect(self.close)
        horizontalLayout = QHBoxLayout()
        horizontalLayout.addWidget(self.contentsWidget)
        horizontalLayout.addWidget(self.pagesWidget, 1)
        buttonsLayout = QHBoxLayout()
        buttonsLayout.addStretch(1)
        buttonsLayout.addWidget(closeButton)
        mainLayout = QVBoxLayout()
        mainLayout.addLayout(horizontalLayout)
        mainLayout.addStretch(1)
        mainLayout.addSpacing(12)
        mainLayout.addLayout(buttonsLayout)
        self.setLayout(mainLayout)
        self.setWindowTitle("Yardım Menüsü")
        self.setWindowIcon(QIcon(':/icon/images/helpwinIcon.png'))
        self.resize(800, 500)
        self.setMinimumWidth(800)

    def changePage(self, current, previous):
        if not current:
            current = previous
        self.pagesWidget.setCurrentIndex(self.contentsWidget.row(current))

    def createIcons(self):
        tanimButton = QListWidgetItem(self.contentsWidget)
        tanimButton.setIcon(QIcon(':/icon/images/description.png'))
        tanimButton.setText("Tanım")
        tanimButton.setTextAlignment(Qt.AlignHCenter)
        tanimButton.setFlags(Qt.ItemIsSelectable | Qt.ItemIsEnabled)
        kullanimSekliButton = QListWidgetItem(self.contentsWidget)
        kullanimSekliButton.setIcon(QIcon(':/icon/images/description2.png'))
        kullanimSekliButton.setText("Kullanım Şekli")
        kullanimSekliButton.setTextAlignment(Qt.AlignHCenter)
        kullanimSekliButton.setFlags(Qt.ItemIsSelectable | Qt.ItemIsEnabled)
        exampleButton = QListWidgetItem(self.contentsWidget)
        exampleButton.setIcon(QIcon(':/icon/images/example.png'))
        exampleButton.setText("Örnekler")
        exampleButton.setTextAlignment(Qt.AlignHCenter)
        exampleButton.setFlags(Qt.ItemIsSelectable | Qt.ItemIsEnabled)
        self.contentsWidget.currentItemChanged.connect(self.changePage)
        self.contentsWidget.setFixedHeight(400)
        self.contentsWidget.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))

if __name__ == '__main__':
    import sys
    app = QApplication(sys.argv)
