import json
import sys
from os import listdir
from PyQt5.QtCore import QMimeData
from PyQt5 import QtWidgets, Qt
from PyQt5.QtGui import QStandardItemModel, QStandardItem
from Components.LeftMenu.Menus.TreeHelpDialog import *
from PyQt5.QtWidgets import QAbstractItemView, QLayout
from configuration import Configuration

childNodes = []


class SceneTreeModel(QStandardItemModel):
    def mimeData(self, indexes):
        name = indexes[0].data()
        mimedata = QMimeData()
        if name in childNodes:
            mimedata.setText(name)
        return mimedata

    def supportedDropActions(self):
        return QtCore.Qt.CopyAction

    def canDropMimeData(self, data, action, row, column, parent):
        return True


class MenuButton(QtWidgets.QPushButton):
    moveSignal = QtCore.pyqtSignal()

    def __init__(self, *args, **kwargs):
        super(MenuButton, self).__init__(*args, **kwargs)
        self.setFixedHeight(50)


class UcLMMenus(QWidget):
    jsonFiles = []
    leftMenuWidth = 115
    descriptions = []
    isTreeOpen = True
    def __init__(self, parent=None):
        super().__init__(parent)

        self.c = Configuration()
        self.parent = parent
        self.setupUi(self)

    # Slide Menu hide -show method
    def hideTreeView(self, control=False):
        if not self.treeView.isVisible() or control:
            self.treeView.hide()
            self.treeView.setVisible(True)
            self.solmenu.setFixedWidth(410)
            return False
        else:
            self.treeView.show()
            self.treeView.setVisible(False)
            self.solmenu.setFixedWidth(115)
            return True

    def solGenislik(self, solmenu=None):
        if solmenu is not None:
            self.solmenu = solmenu



    def expanded(self, index):
        dosyaAdi = index.data()  # type: object
        if dosyaAdi in self.descriptions:
            helpDialog = TreeHelpDialog(dosyaAdi)
            helpDialog.exec_()

    def ReadFromFile(self, dosya):
        self.c = Configuration()
        dosyaPath = self.c.getHomeDir() + self.c.getTreeMenuPath("tree_menu_path") + dosya
        with open(dosyaPath, encoding='utf-8') as f:
            data = json.load(f)
        model = SceneTreeModel()
        self.TreeViewFill(model, data)
        self.treeView.setModel(model)

    def TreeViewFill(self, model, data):
        for root, children in data.items():
            parent = QStandardItem(root)
            self.AddChild(children, parent)
            model.appendRow(parent)

    def AddChild(self, children, parent):
        if isinstance(children, dict):
            for item, items in children.items():
                child = QStandardItem(item)
                if parent.text() in self.descriptions:
                    parent.setIcon(QIcon(':/icon/images/treeview-help.png'))
                if item in self.descriptions:
                    child.setIcon(QIcon(':/icon/images/treeview-help.png'))
                parent.appendRow(child)
                self.AddChild(items, child)
        elif children != None:  # if there is one node
            node = QStandardItem(children)
            node.setToolTip("Bu kodu editöre sürükle-bırak şeklinde taşıyabilirsiniz")
            childNodes.append(children)
            parent.appendRow(node)

    # endregion

    @QtCore.pyqtSlot()
    def moveUp(self):
        ix = self.listWidget.moveCursor(QtWidgets.QAbstractItemView.MoveUp, QtCore.Qt.NoModifier)
        self.listWidget.setCurrentIndex(ix)
        if 1 < self.activeMenu:
            self.toolButtons[self.activeMenu].setStyleSheet(
                "background-color: #394b58; color: white; font: 10pt; padding-top:10px;")
            self.previousActiveMenu = self.activeMenu
            self.activeMenu = self.activeMenu - 1
            self.toolButtons[self.activeMenu].setStyleSheet(
                "background-color: #6b899f; color: white; font: 10pt; padding-top:10px;")
            self.MenuActionClick(self.jsonFiles[self.activeMenu - 1], self.activeMenu)
            self.hideTreeView(True)

    @QtCore.pyqtSlot()
    def moveDown(self):

        if self.menuItemCount > self.activeMenu:
            self.toolButtons[self.activeMenu].setStyleSheet(
                "background-color: #394b58; color: white; font: 10pt; padding-top:10px;")
            self.previousActiveMenu = self.activeMenu
            self.activeMenu = self.activeMenu + 1
            self.toolButtons[self.activeMenu].setStyleSheet(
                "background-color: #6b899f; color: white; font: 10pt; padding-top:10px;")
            self.listWidget.setCurrentRow(self.activeMenu - 1)
            self.MenuActionClick(self.jsonFiles[self.activeMenu - 1], self.activeMenu)
            self.hideTreeView(True)

    def FillMenuCategories(self):
        self.c = Configuration()
        dosya = 'menus.json'
        dosyaPath = self.c.getHomeDir() + self.c.getTreeMenuPath("tree_menu_path") + dosya

        with open(dosyaPath, encoding='utf-8') as f:
            leftMenuJson = json.load(f)
        i = 1
        self.menuItemCount = len(leftMenuJson.items())
        for root, children in leftMenuJson.items():
            item = QtWidgets.QListWidgetItem()
            widget = QWidget()
            jsonfile = children['jsonfile']
            self.jsonFiles.append(children['jsonfile'])
            self.toolButtons[i] = toolButton = QtWidgets.QToolButton()
            toolButton.setStyleSheet(
                "QToolButton {background-color: #394b58; color: white; font: 10pt; padding-top:10px;}\n"
                "QToolButton:hover {background-color: #6b899f; color: white; font: 10pt; padding-top:10px;}\n")
            toolButton.setFixedWidth(115)
            toolButton.setFixedHeight(115)
            self.toolButtons[1].setStyleSheet("background-color: #6b899f; color: white; font: 10pt; padding-top:10px;")
            self.previousActiveMenu = 1
            self.activeMenu = 1
            icon = QtGui.QIcon()
            icon.addPixmap(QtGui.QPixmap(children['icon']), QtGui.QIcon.Normal, QtGui.QIcon.Off)
            toolButton.setIconSize(QtCore.QSize(40, 40))
            toolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
            toolButton.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))
            toolButton.setIcon(icon)
            toolButton.setText(root)
            self.toolButtons[i].clicked.connect(
                lambda checked, index=i, jsonfile=jsonfile: self.MenuActionClick(jsonfile, index))
            toolButton.setObjectName("toolButton")
            toolButton.raise_()
            widgetLayout = QHBoxLayout()
            widgetLayout.addWidget(toolButton)
            widgetLayout.addStretch()
            widgetLayout.setSizeConstraint(QLayout.SetFixedSize)
            widgetLayout.setContentsMargins(0, 0, 0, 0)
            widget.setLayout(widgetLayout)
            item.setSizeHint(widget.sizeHint())
            self.listWidget.addItem(item)
            self.listWidget.setItemWidget(item, widget)
            i = i + 1

    def MenuActionClick(self, jsonFile, index):
        if self.activeMenu != 0:
            self.toolButtons[self.activeMenu].setStyleSheet(
                "background-color: #394b58; color: white; font: 10pt; padding-top:10px;")
        self.toolButtons[index].setStyleSheet("background-color: #6b899f; color: white; font: 10pt; padding-top:10px;")
        self.ReadFromFile(jsonFile)
        self.previousActiveMenu = self.activeMenu
        self.activeMenu = index

        if self.activeMenu == self.previousActiveMenu:
            self.hideTreeView()
        else:
            self.hideTreeView(True)


    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))

    def setupUi(self, Form):

        self.treeView = QtWidgets.QTreeView(self.parent)
        # slide  show - hide
        self.treeView.hide()
        self.treeView.setVisible(False)


        self.downButton = MenuButton(icon=QIcon(':/icon/images/down.png'))
        self.upButton = MenuButton(icon=QIcon(':/icon/images/up.png'))
        self.toolButtons = {}

        Form.setObjectName("Form")
        self.frame_solmenu = QtWidgets.QFrame(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Expanding)
        self.frame_solmenu.setSizePolicy(sizePolicy)
        self.frame_solmenu.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_solmenu.setObjectName("frame_solmenu")

        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setObjectName("gridLayout")
        self.gridLayout.addWidget(self.frame_solmenu, 0, 0, 1, 1)

        self.leftMenuLayout = QtWidgets.QVBoxLayout(self.frame_solmenu)
        self.leftMenuLayout.setObjectName("leftMenuLayout")
        self.leftMenuLayout.setContentsMargins(0, 0, 0, 0)

        self.upButton.setIconSize(QSize(65, 65))
        self.upButton.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))
        self.upButton.moveSignal.connect(self.moveUp)
        self.upButton.setFixedWidth(self.leftMenuWidth)
        self.upButton.setFixedHeight(42)
        self.upButton.setStyleSheet("QPushButton {background-color: #394b58; border:none}"
                                    "QPushButton:hover {background-color: #6b899f; border:none}")
        self.downButton.setIconSize(QSize(65, 65))
        self.downButton.setFixedWidth(self.leftMenuWidth)
        self.downButton.setFixedHeight(50)
        self.downButton.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))
        self.downButton.moveSignal.connect(self.moveDown)
        self.downButton.setStyleSheet("QPushButton {background-color: #394b58; border:none; margin-bottom:5px}"
                                      "QPushButton:hover {background-color: #6b899f; border:none} ")

        self.listWidget = QtWidgets.QListWidget(self.frame_solmenu)
        self.listWidget.setStyleSheet("background-color: #394b58; border:none; ")
        self.listWidget.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.listWidget.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.listWidget.setFixedWidth(self.leftMenuWidth)

        self.FillMenuCategories()

        self.listWidget.setCurrentRow(self.activeMenu)

        # self.MenuActionClick(self.jsonFiles[self.activeMenu - 1], self.activeMenu)

        self.leftMenuLayout.addWidget(self.upButton)
        self.leftMenuLayout.addWidget(self.listWidget)
        self.leftMenuLayout.addWidget(self.downButton)

        self.downButton.clicked.connect(self.moveDown)
        self.upButton.clicked.connect(self.moveUp)

        self.treeView.setObjectName("treeView")
        self.treeView.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.treeView.setMinimumSize(QtCore.QSize(290, 400))
        self.treeView.setMaximumSize(QtCore.QSize(290, 16777215))
        self.gridLayout.addWidget(self.treeView, 0, 1, 1, 1)
        self.treeView.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))
        self.treeView.setStyleSheet("QTreeView {\n"
                                    "    background-color: #6b899f;\n"
                                    "    border: 1px solid;\n"
                                    "    border-color: rgb(255, 255, 255);\n"
                                    "    padding: 10px;\n"
                                    "    padding-top: -20px;\n"
                                    "    outline: 0px;\n"
                                    "    font:   rgb(255, 255, 255);\n"
                                    "    font:  10pt \"MS Shell Dlg 2\";\n"
                                    "    selection-background-color: transparent;\n"
                                    "}\n"
                                    "\n"
                                    "QTreeView::item {\n"
                                    "         border: 3px solid  rgb(0, 170, 255);                \n"
                                    "         height: 50px;                            \n"
                                    "         padding-left: 20px;                      \n"
                                    "         margin:10px;       \n"
                                    "         background-color: rgb(255, 255, 255);\n"
                                    "         color: rgb(0, 170, 255);      \n"
                                    "         selection-background-color: transparent;\n"
                                    "    }\n"
                                    "\n"
                                    "QTreeView::item::hover {\n"
                                    "       background-color:  #b9e6fb; \n"
                                    "}\n"
                                    "\n"
                                    "QTreeView::branch:has-siblings:!adjoins-item {\n"
                                    "    border-image: url(:/icon/images/vline.png) 0;\n"
                                    "    background-color: transparent;\n"
                                    "}\n"
                                    "\n"
                                    "QTreeView::branch:has-siblings:adjoins-item {\n"
                                    "    border-image: url(:/icon/images/branch-more.png) 0;\n"
                                    "    background-color: transparent;\n"
                                    "}\n"
                                    "\n"
                                    "QTreeView::branch:!has-children:!has-siblings:adjoins-item {\n"
                                    "    border-image: url(:/icon/images/branch-end.png) 0;\n"
                                    "    background-color: transparent;\n"
                                    "}\n"
                                    "\n"
                                    "QTreeView::branch:has-children:!has-siblings:closed,\n"
                                    "QTreeView::branch:closed:has-children:has-siblings {\n"
                                    "        border-image: none;\n"
                                    "        image: url(:/icon/images/branch-closed.png);\n"
                                    "        background-color: transparent;\n"
                                    "}\n"
                                    "\n"
                                    "QTreeView::branch:open:has-children:!has-siblings,\n"
                                    "QTreeView::branch:open:has-children:has-siblings  {\n"
                                    "        border-image: none;\n"
                                    "        image: url(:/icon/images/branch-open.png);\n"
                                    "        background-color: transparent;\n"
                                    "}\n"
                                    "\n"
                                    "QTreeView::item:!has-children{\n"
                                    "\n"
                                    "      border-radius: 0px 0px 0px 0px;\n"
                                    "      border:3px solid  #e7a61a;  \n"
                                    "      height: 50px;\n"
                                    "      padding-left: 20px;                      \n"
                                    "      margin:10px;                      \n"
                                    "      background-color: rgb(255, 255, 255);        \n"
                                    "      color: #e7a61a;\n"
                                    "\n"
                                    "}\n"
                                    "QTreeView::item:!has-children::hover {\n"
                                    "      border-radius: 0px 0px 0px 0px;\n"
                                    "      border: 3px solid #e7a61a;\n"
                                    "      height: 50px;                        \n"
                                    "      padding-left: 20px;                      \n"
                                    "      margin:10px;                      \n"
                                    "      background-color: #e7a61a; \n"
                                    "      color: rgb(255, 255, 255); \n"
                                    "}\n"
                                    "\n"
                                    "\n"
                                    "")
        self.treeView.setItemsExpandable(True)
        self.treeView.setDragDropMode(QAbstractItemView.DragOnly)
        self.treeView.setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.treeView.setHeaderHidden(True)
        self.treeView.clicked.connect(self.expanded)

        dosyaPath = self.c.getHomeDir() + self.c.getHtmlHelpPath("html_help_path")
        self.descriptions = [f for f in listdir(dosyaPath) if not os.path.isdir(f)]
        self.ReadFromFile('temelkomutlar.json')
        self.show()