# configuration.py

import configparser
import os
import sys

class Configuration():
    
    def __init__(self):
        
        self.config = configparser.ConfigParser()
        file = self.getDir()
        
        self.config.read(file)

    def getHomeDir(self):
        path = os.path.realpath(__file__)
        basename = self.checkPath(os.path.dirname(path))
        return basename+ '/'

    def getDir(self):
        path = os.path.realpath(__file__)      # Pfad ermitteln
        
        basename = self.checkPath(os.path.dirname(path))
        path = basename + '/Config/pynar.ini'
        
        return path

    # def getHelloWorldDir(self):
    #     path = os.path.realpath(__file__)  # Pfad ermitteln
    #
    #     basename = self.checkPath(os.path.dirname(path))
    #     path = basename + '/merhabadunya.py'
    #
    #     return path

    def getRun(self, system):
        return self.config['Run'][system]
    
    def getTerminal(self, system):
        return self.config['Terminal'][system]
    
    def getInterpreter(self, system):
        return self.config['Interpreter'][system]
    
    def getSystem(self):
        return self.config['System']['system']
    
    def getTab(self):
        return self.config['Tab']['tab']
    
    def getFontSize(self):
        return self.config['Size']['size']

    # MSTF: Log bilgileri
    def getLogEnabled(self):
        return self.config['Logging']['logging']

    def getLogFolder(self):
        return self.config['Logging']['Log_folder']
    #
    def setSystem(self, system):
        self.config['System']['system'] = system

        path = os.path.realpath(__file__)    
        basename = self.checkPath(os.path.dirname(path))

        iniPath = basename + "/Config/pynar.ini"
        
        with open(iniPath, 'w', encoding='utf8') as f:
            self.config.write(f)

    def updateConfig(self,selector,param,value):
        self.config[selector][param] = value

        path = os.path.realpath(__file__)
        basename = self.checkPath(os.path.dirname(path))

        iniPath = basename + "/Config/pynar.ini"

        with open(iniPath, 'w', encoding='utf8') as f:
            self.config.write(f)

    def setStandard(self):
        config = configparser.ConfigParser()
        
        config['Run'] = {}
        config['Run']['mate'] = 'mate-terminal -x sh -c "python3 {}; exec bash"'
        config['Run']['pardus'] = 'xfce4-terminal -x sh -c "python3 {}; exec bash"'
        config['Run']['gnome'] = 'gnome-terminal -- sh -c "python3 {}; exec bash"'
        config['Run']['kde'] = 'konsole --hold -e "python3 {}"'
        config['Run']['xterm'] = 'xterm -hold -e "python3 {}"'
        config['Run']['windows'] = 'start cmd.exe /k "TITLE PYNAR CALISTIR & color 0F & prompt $G & python {} & echo. &(Pause >nul | echo Devam icin bir tusa basiniz...) & exit"'
        config['Run']['mac'] = 'open -a Terminal ./python3 {}'
        
        config['Terminal'] = {}
        config['Terminal']['mate'] = 'mate-terminal'
        config['Terminal']['pardus'] = 'xfce4-terminal'
        config['Terminal']['gnome'] = 'gnome-terminal'
        config['Terminal']['kde'] = 'konsole'
        config['Terminal']['xterm'] = 'xterm'
        config['Terminal']['windows'] = 'start cmd'
        config['Terminal']['mac'] = 'open -a Terminal ./' 
        
        config['Interpreter'] = {}
        config['Interpreter']['mate'] = 'mate-terminal -x "python3"'
        config['Interpreter']['pardus'] = 'xfce4-terminal -x "python3"'
        config['Interpreter']['gnome'] = 'gnome-terminal -- "python3"'
        config['Interpreter']['kde'] = 'konsole -e python3'
        config['Interpreter']['xterm'] = 'xterm python3'
        config['Interpreter']['windows'] = 'start cmd /K "TITLE PYTHON YORUMLAYICI & python"'
        config['Interpreter']['mac'] = 'open -a Terminal ./python3'
        
        config['System'] = {}
        config['System']['system'] = ''
        
        config['Tab'] = {}
        config['Tab']['tab'] = '4'
        
        config['Size'] = {}
        config['Size']['size'] = '16'

        config['JsonFiles'] = {}
        config['JsonFiles']['recentjson'] = 'Data/recents.json'

        config['PreferenceRecent'] = {}
        config['PreferenceRecent']['prefrecent'] = 'first'

        config['TreeMenuFiles'] = {}
        config['TreeMenuFiles']['tree_menu_path'] = 'Data/TreeMenuFiles/'

        config['HtmlHelpFiles'] = {}
        config['HtmlHelpFiles']['html_help_path'] = 'Data/HtmlHelpFiles/'

        config['Logging'] = {}
        config['Logging']['logging'] = 'True'
        config['Logging']['log_folder'] = 'Log'

        return config


    def checkPath(self, path):
        if '\\' in path:
            path = path.replace('\\', '/')
        return path


    def getJsonPath(self, param):
        return self.config['JsonFiles'][param]

    def getTreeMenuPath(self, param):
        return self.config['TreeMenuFiles'][param]

    def getHtmlHelpPath(self, param):
        return self.config['HtmlHelpFiles'][param]

    def getParam(self,selector, param):
        return self.config[selector][param]
    
if __name__ == '__main__':
    c = Configuration()

    system = c.getSystem()
    runCommand = c.getRun(system)
    terminalCommand = c.getTerminal(system)
    interpreterCommand = c.getInterpreter(system)

    #c.setSystem('gnome')

    print(system + ':\n' + runCommand + '\n' + terminalCommand + '\n' + interpreterCommand)
    
