import sys
import os
from PyQt5 import Qt
from PyQt5.QtCore import QSize, QRect, QThread, pyqtSignal
from PyQt5.QtWidgets import (QApplication, QWidget, QDialog, QHBoxLayout,
                             QVBoxLayout, QGridLayout, QLabel, QLineEdit,
                             QPushButton, QMainWindow, QCheckBox, QDesktopWidget,
                             QGroupBox, QSpinBox, QTextEdit, QTabWidget,
                             QDialogButtonBox, QMessageBox, QListWidget,
                             QListWidgetItem, QComboBox)
from PyQt5.QtGui import (QFont, QPalette, QIcon)
from PyQt5.Qt import Qt
from configuration import Configuration
from Components.DetectOs.detectos import detectos
from widgets import (MessageBox, Label, RadioButton, PushButton,
                     ListWidget, WhiteLabel, TabWidget, TextEdit)
from deadcodechecker import DeadCodeChecker
from pycodechecker import PyCodeChecker
from natsort import natsorted
import urllib
from urllib import parse
import requests
import subprocess
import re

class Dialog(QDialog):

    def __init__(self, parent=None, textPad=None):
        super().__init__()
        palette = QPalette()
        role = QPalette.Background
        # palette.setColor(role, QColor('#2c2c2c'))
        self.setGeometry(parent.x() + 200 , parent.y() + 100, 500, 400)
        self.setPalette(palette)
        self.mainWindow = parent



class SettingsDialog(Dialog):
    outSignal = pyqtSignal(str)
    okSignal = pyqtSignal(str, bool)

    def __init__(self, parent=None, textPad=None):
        super().__init__(parent, textPad)
        self.parent = parent
        self.textPad = textPad
        self.setWindowTitle('Ayarlar')
        self.setWindowIcon(QIcon(':/icon/images/headerLogo1.png'))
        self.setStyleSheet("background-color: #CAD7E0;")
        self.setMinimumSize(QSize(620, 383))
        self.setMaximumSize(QSize(620, 383))
        self.baseUrl = 'https://pypi.org/pypi/'
        self.headers = {'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:78.0) Gecko/20100101 Firefox/78.0'}
        self.initUI()

    def initUI(self):
        font = QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(10)

        self.tabWidget = QTabWidget()
        self.tabWidget.setGeometry(QRect(0, 0, 595, 353))
        self.tabWidget.setFont(font)
        self.tabWidget.setStyleSheet(" background-color: rgb(255, 255, 255);")

        Settings = QWidget()

        groupBox = QGroupBox(Settings)
        groupBox.setGeometry(QRect(20, 5, 550, 307))
        groupBox.setTitle('Kod Editör')
        groupBox.setFont(font)

        self.c = Configuration()


        label1 = Label(groupBox)
        label1.setGeometry(QRect(30, 50, 260, 22))
        label1.setText('Beyaz boşluklarda <Tab> genişliği:')
        label1.setFont(font)

        self.tabWidthBox = QSpinBox(groupBox)
        self.tabWidthBox.setMinimum(2)
        self.tabWidthBox.setMaximum(10)
        self.tabWidthBox.setGeometry(QRect(300, 51, 42, 22))
        self.tabWidthBox.setFont(font)
        tab = int(self.c.getTab())
        self.tabWidthBox.setValue(tab)

        label2 = Label(groupBox)
        label2.setGeometry(QRect(30, 90, 255, 22))
        label2.setText('Editör Yazı Tipi Boyutu:')
        label2.setFont(font)

        self.fontSizeBox = QSpinBox(groupBox)
        self.fontSizeBox.setMinimum(6)
        self.fontSizeBox.setMaximum(30)
        self.fontSizeBox.setGeometry(QRect(300, 90, 42, 22))
        self.fontSizeBox.setFont(font)
        fontSize = int(self.c.getFontSize())
        self.fontSizeBox.setValue(fontSize)

        self.loggingCase = QCheckBox(groupBox)
        self.loggingCase.setGeometry(QRect(224, 137, 231, 20))
        self.loggingCase.setStyleSheet("color:rgb(82, 95, 99)")
        self.loggingCase.setFont(font)
        self.loggingCase.setCheckState(0 if self.c.getParam('Logging', 'logging') == "False" else 1)
        self.loggingCase.setTristate(0)

        labelLogging = WhiteLabel(groupBox)
        labelLogging.setGeometry(QRect(30, 135, 185, 22))
        labelLogging.setText('Kullanım verilerini kaydet')
        labelLogging.setFont(font)

        label3 = WhiteLabel(groupBox)
        label3.setGeometry(QRect(30, 205, 570, 22))
        label3.setText('*Bu ayarlarda yaptığınız değişiklikler Pynar Editörünü tekrar')
        label3.setStyleSheet("color: rgb(255, 0, 0);")
        label3.setFont(font)

        label4 = WhiteLabel(groupBox)
        label4.setGeometry(QRect(30, 223, 401, 22))
        label4.setText('  başlattığınızda geçerli olacaktır.')
        label4.setStyleSheet("color: rgb(255, 0, 0);")
        label4.setFont(font)


        
        okButton = PushButton(Settings)
        okButton.setGeometry(QRect(50, 260, 141, 41))
        okButton.setText('TAMAM')
        okButton.setFont(font)
        okButton.setStyleSheet("QPushButton { color: white;padding: 5px;font-size: 14px;margin: 4px 2px;border-radius: 4px; background-color: rgb(0, 170, 255);} " \
                      "QPushButton::hover{background-color:rgb(4, 124, 184)}")


        self.tabWidget.addTab(Settings,"Editör Ayarları")


        Package_manager = QWidget()
        Package_manager.setFont(font)

        self.Description_line = QTextEdit(Package_manager)
        self.Description_line.setReadOnly(True)
        self.Description_line.setGeometry(QRect(300, 70, 271, 201))
        self.Description_line.setFont(font)
        self.Description_line.setStyleSheet("background-color: rgb(255, 203, 201);")
        self.Description_line.setText("Paketler pypi.org siyesinden indirilmektedir. "
                                      "Aradığınız paket pypi.org sitesinde bulunduktan "
                                      "sonra paket bilgisi kısmında paketin bilgileri görüntülenir. "
                                      "Aradığınız pakete ait son sürümü yerine farklı bir sürümünü "
                                      "indirmek istiyorsanız sürüm numarası seç bölümünden paketinize "
                                      "ait tüm sürüm numaraları arasından seçiminizi yapabilirsiniz.")

        self.Search_line = QLineEdit(Package_manager)
        self.Search_line.setGeometry(QRect(10, 10, 390, 31))
        self.Search_line.setFont(font)
        self.Search_line.setPlaceholderText("Aradığınız paketin tam adını yazınız.")

        self.Search_button = QPushButton(Package_manager)
        self.Search_button.setGeometry(QRect(410, 5, 141, 41))
        self.Search_button.setText("PyPI\'dan Paket Bul")
        self.Search_button.setFont(font)
        self.Search_button.setStyleSheet("QPushButton { color: white;padding: 5px;font-size: 14px;margin: 4px 2px;border-radius: 4px; background-color: rgb(0, 170, 255);} " \
                      "QPushButton::hover{background-color:rgb(4, 124, 184)}")

        self.Download_button = QPushButton(Package_manager)
        self.Download_button.setGeometry(QRect(440, 277, 141, 41))
        self.Download_button.setText("İndir")
        self.Download_button.setEnabled(False)
        self.Download_button.setFont(font)
        self.Download_button.setStyleSheet("QPushButton { color: white;padding: 5px;font-size: 14px;margin: 4px 2px;border-radius: 4px; background-color: rgb(0, 170, 255);} " \
                      "QPushButton::hover{background-color:rgb(4, 124, 184)}")

        self.Version_list = QComboBox(Package_manager)
        self.Version_list.setStyleSheet("QComboBox { border : 1px solid gray;}")
        self.Version_list.setGeometry(QRect(170, 285, 121, 22))
        self.Version_list.setFont(font)

        self.Update_button = QPushButton(Package_manager)
        self.Update_button.setGeometry(QRect(299, 277, 141, 41))
        self.Update_button.setText("Güncelle")
        self.Update_button.setEnabled(False)
        self.Update_button.setFont(font)
        self.Update_button.setStyleSheet("QPushButton { color: white;padding: 5px;font-size: 14px;margin: 4px 2px;border-radius: 4px; background-color: rgb(0, 170, 255);} " \
                      "QPushButton::hover{background-color:rgb(4, 124, 184)}")

        self.List_line = QListWidget(Package_manager)
        self.List_line.setGeometry(QRect(10, 70, 281, 201))
        self.List_line.setFont(font)

        Label_vers = QLabel(Package_manager)
        Label_vers.setGeometry(QRect(10, 285, 155, 20))
        Label_vers.setText("Sürüm Numarası Seç")
        Label_vers.setFont(font)

        Label_desp = QLabel(Package_manager)
        Label_desp.setGeometry(QRect(300, 48, 261, 20))
        Label_desp.setText("Paket Bilgisi")
        Label_desp.setFont(font)

        Label_list = QLabel(Package_manager)
        Label_list.setGeometry(QRect(10, 48, 261, 20))
        Label_list.setText("Paket Listesi")
        Label_list.setFont(font)

        self.tabWidget.addTab(Package_manager, "Paket Yöneticisi")

        self.dPackageDict = self._setInstructıonPage()

        okButton.pressed.connect(self.close)
        self.List_line.itemClicked.connect(lambda: self._listItemClicked())
        self.Search_line.returnPressed.connect(lambda: self._searchQuery(self.Search_line))
        self.Search_button.clicked.connect(lambda: self._searchQuery(self.Search_line))
        self.Download_button.clicked.connect(lambda: self._btnClicked())
        self.Version_list.currentIndexChanged.connect(lambda: self._btnVersion())
        self.Update_button.clicked.connect(lambda: self._updatePackage())
        self.outSignal.connect(self._setDescText)
        self.okSignal.connect(self._updateList)
        self.loggingCase.stateChanged.connect(self.logging)

        self.btnIsUninstall = False
        self.dwnVersion = ""

        layout = QVBoxLayout()
        layout.addWidget(self.tabWidget)
        self.setLayout(layout)
        self.center()
        self.show()

    def logging(self):
        cs = bool(self.loggingCase.checkState())
        self.c.updateConfig('Logging', 'logging', str(cs))


    def _btnVersion(self):
        if self.btnIsUninstall:
            if self.dwnVersion == self.Version_list.currentText():
                self.Update_button.setEnabled(False)
            else:
                self.Update_button.setEnabled(True)

    def _btnClicked(self):
        if self.btnIsUninstall:
            self._uninstallPackage()
        else:
            self._downloadPackage()

    def _btnChanged(self):
        self.Download_button.setEnabled(False)
        self.btnIsUninstall = False
        self.Download_button.setText("İndir")

    def _setInstructıonPage(self):
        import pkg_resources, imp

        self.List_line.clear()
        imp.reload(pkg_resources)
        installed_packages = pkg_resources.working_set
        installed_packages_key_list = tuple((["%s" % (i.key)
                                              for i in installed_packages]))
        installed_packages_version_list = tuple((["%s" % (i.version)
                                                  for i in installed_packages]))
        for i in installed_packages_key_list:
            self.List_line.addItem(i)
        self.List_line.sortItems()
        dPackageDict = dict(zip(installed_packages_key_list, installed_packages_version_list))
        dPackageDict['isInstalled'] = True
        return dPackageDict

    def _listItemClicked(self):
        self.Description_line.setStyleSheet("background-color: rgb(255, 255, 255);")
        try:
            packageName = self.List_line.currentItem().text()
            jsonDataUrl = self.baseUrl + urllib.parse.quote(
                packageName) + "/json"
            packageURL = "https://pypi.org/project/" + packageName + "/"
            self.dPackageDict['Json page'] = jsonDataUrl
            self.dPackageDict['PyPI page'] = packageURL
            self.dPackageDict['isInstalled'] = True
            self._fetchPackageData()
        except Exception as e:
            print(e)
            pass

    def _fetchPackageData(self):
        import json

        try:
            req = requests.get(self.dPackageDict['Json page'], headers=self.headers)
            jsonData = req.json()
            self.Version_list.clear()
            for i in natsorted(list(jsonData['releases'].keys()))[::-1]:
                self.Version_list.addItem(i)
        except Exception as e:
            print(e)
            self._packageNotFound()
            return False
        else:
            self.dPackageDict['Author'] = jsonData['info']['author']
            self.dPackageDict['name'] = jsonData['info']['name']
            self.dPackageDict['version'] = jsonData['info']['version']
            self.dPackageDict['details'] = jsonData['info']['summary']
            self.dPackageDict['Homepage'] = jsonData['info']['home_page']
            self.dPackageDict['Requirements'] = jsonData['info']['requires_dist']
            for data in self.dPackageDict:
                if self.dPackageDict[data] == None:
                    self.dPackageDict[data] = ""
            self._writeData(self.dPackageDict['isInstalled'])
            return True

    def _writeData(self, isInstalled):
        if isInstalled == True:
            pName = self.dPackageDict['name']
            try:
                tempList = pName.split("_")
            except Exception as e:
                print(e)
            else:
                pName = "-".join(tempList)
                self.Description_line.setText(f"{str(self.dPackageDict['name']).upper()}")
                self.Description_line.append("\n")
                self.Description_line.append(f"İndirilen versiyon : {self.dPackageDict[pName.lower()]}")
                self.Description_line.append("\n")
                self.Description_line.append(f"Güncel versiyon : {self.dPackageDict['version']} ")
                self.Description_line.append(f"Açıklama :{self.dPackageDict['details']} ")
                self.Description_line.append(f"Anasayfa : {self.dPackageDict['Homepage']}  ")
                self.Description_line.append(f"PyPI sayfası :{self.dPackageDict['PyPI page']} ")
                self.Description_line.append(f"Yazar : {self.dPackageDict['Author']} ")
                self.Description_line.append(f"Gerekli olanlar : {str(self.dPackageDict['Requirements'])}")
                try:
                    cursor = self.Description_line.textCursor()
                    cursor.movePosition(0)
                    self.Description_line.setTextCursor(cursor)
                except Exception as e:
                    print(e)
                    pass
                self.dwnVersion = self.dPackageDict[pName.lower()]
                self.btnIsUninstall = True
                self.Version_list.setCurrentIndex(self.Version_list.findText(self.dPackageDict[pName.lower()]))
                self._btnVersion()
                self.Download_button.setEnabled(True)
                self.Download_button.setText("Kaldır")
                self.Description_line.setStyleSheet("background-color: rgb(255, 255, 255);")
        else:
            self.Description_line.setText(f"{str(self.dPackageDict['name']).upper()}")
            self.Description_line.append("\n")
            self.Description_line.append(f"Güncel versiyon : {self.dPackageDict['version']} ")
            self.Description_line.append(f"Açıklama :{self.dPackageDict['details']} ")
            self.Description_line.append(f"Anasayfa : {self.dPackageDict['Homepage']}  ")
            self.Description_line.append(f"PyPI sayfası :{self.dPackageDict['PyPI page']} ")
            self.Description_line.append(f"Yazar : {self.dPackageDict['Author']} ")
            self.Description_line.append(f"Gerekli olanlar : {str(self.dPackageDict['Requirements'])}")
            try:
                cursor = self.Description_line.textCursor()
                cursor.movePosition(0)
                self.Description_line.setTextCursor(cursor)
            except Exception as e:
                print(e)
                pass
            self.btnIsUninstall = False
            self.Download_button.setEnabled(True)
            self.Update_button.setEnabled(False)
            self.Download_button.setText("İndir")
            self.Description_line.setStyleSheet("background-color: rgb(255, 255, 255);")
        return

    def _searchQuery(self, query):
        if query.text().strip() == "":
            return
        packageName = query.text().strip().lower()
        findList = self.List_line.findItems(packageName, Qt.MatchExactly)

        packageURL = "https://pypi.org/project/" + packageName + "/"
        jsonDataUrl = self.baseUrl + urllib.parse.quote(packageName) + "/json"
        self.dPackageDict['PyPI page'] = packageURL
        self.dPackageDict['Json page'] = jsonDataUrl

        if findList:
            self.dPackageDict['isInstalled'] = True
        else:
            self.dPackageDict['isInstalled'] = False

        isFind = self._fetchPackageData()
        if isFind:
            if findList:
                self.List_line.setCurrentItem(findList[0])
            else:
                self.List_line.setCurrentRow(-1)
                self.Version_list.setCurrentIndex(0)
        else:
            pass

    def _downloadPackage(self):
        from threading import Thread
        self.block()
        self.List_line.setCurrentRow(0)
        self.Description_line.clear()
        runs = Thread(target=self.download_command)
        runs.start()

    def download_command(self, update=False):
        self.thread = QThread()
        self.listThread = QThread()
        text = (self.Search_line.text() if not update else self.List_line.currentItem().text())
        version = "==" + self.Version_list.currentText()
        process = subprocess.Popen([sys.executable, '-m', 'pip', 'install', text + version], stdout=subprocess.PIPE)
        while True:
            output = process.stdout.readline()
            if output:
                self.outSignal[str].emit(output.strip().decode() + "\n")
            rc = process.poll()
            if rc == 1:
                self.thread.started.connect(self._downloadErr if not update else self._updateErr)
                self.thread.start()
                return
            elif rc == 0:
                self.okSignal[str, bool].emit(text, True)
                self.listThread.started.connect(self._listItemClicked)
                self.thread.started.connect(self._downloadOk if not update else self._updateOk)
                self.listThread.start()
                self.thread.start()
                return
            elif rc != None:
                print(rc)
                return

    def _setDescText(self, text):
        self.Description_line.insertPlainText(text)
        try:
            cursor = self.Description_line.textCursor()
            cursor.movePosition(len(self.Description_line.toPlainText()) - 1)
            self.Description_line.setTextCursor(cursor)
        except Exception as e:
            print(e)
            pass

    def _updateList(self, text, isDwn):
        self.prev_dPackageDict = self.dPackageDict
        self.dPackageDict = self._setInstructıonPage()
        if isDwn:
            try:
                self.List_line.setCurrentItem(self.List_line.findItems(text, Qt.MatchExactly)[0])
            except Exception as e:
                text = list(self.dPackageDict.keys() - self.prev_dPackageDict.keys())[0]
                self.List_line.setCurrentItem(self.List_line.findItems(text, Qt.MatchExactly)[0])
                print(e)
                pass

    def _uninstallPackage(self):
        from threading import Thread
        self.block()

        self.Description_line.clear()
        runs = Thread(target=self.uninstall_command)
        runs.start()

    def uninstall_command(self):
        import subprocess
        process = subprocess.Popen(
            [sys.executable, '-m', 'pip', 'uninstall', '-y', self.List_line.currentItem().text()],
            stdout=subprocess.PIPE)
        while True:
            output = process.stdout.readline()

            if output:
                self.outSignal[str].emit(output.strip().decode() + "\n")
            rc = process.poll()
            if rc == 1:
                self.thread = QThread()
                self.thread.started.connect(self._uninstallErr)
                self.thread.start()
                break
            elif rc == 0:
                self.okSignal[str, bool].emit("", False)
                self.thread = QThread()
                self.thread.started.connect(self._uninstallOk)
                self.thread.start()
                self.Version_list.clear()
                break
            elif rc != None:
                print(rc)
                break
        self._btnChanged()

    def _updatePackage(self):
        from threading import Thread
        self.block()

        self.Description_line.clear()
        runs = Thread(target=self.download_command, args=[True])
        runs.start()

    def block(self):
        self.Search_button.setEnabled(False)
        self.Search_line.setEnabled(False)
        self.List_line.setEnabled(False)
        self.Download_button.setEnabled(False)
        self.Update_button.setEnabled(False)

    def noneBlock(self):
        self.Search_button.setEnabled(True)
        self.Search_line.setEnabled(True)
        self.List_line.setEnabled(True)

    def _packageNotFound(self):
        messageBox = QMessageBox(QMessageBox.Information, "text", "text")
        messageBox.setWindowIcon(QIcon(':/icon/images/headerLogo1.png'))
        messageBox.setStandardButtons(QMessageBox.Ok)
        messageBox.setWindowTitle("Hata")
        messageBox.setText("Paket Bulunamadı!")
        messageBox.setIcon(QMessageBox.Warning)
        messageBox.exec()
        self.noneBlock()
        return

    def _downloadErr(self):
        messageBox = QMessageBox(QMessageBox.Warning, "text", "text")
        messageBox.setWindowIcon(QIcon(':/icon/images/headerLogo1.png'))
        messageBox.setStandardButtons(QMessageBox.Ok)
        messageBox.setWindowTitle("Yükleme Hatası")
        messageBox.setText("İndirme Başarısız !!!")
        messageBox.exec()
        self.noneBlock()
        return

    def _downloadOk(self):
        messageBox = QMessageBox(QMessageBox.Information, "text", "text")
        messageBox.setWindowIcon(QIcon(':/icon/images/headerLogo1.png'))
        messageBox.setStandardButtons(QMessageBox.Ok)
        messageBox.setWindowTitle("Yükleme Tamamlandı")
        messageBox.setText("İndirme Başarılı !!!")
        messageBox.exec()
        self.noneBlock()
        return

    def _uninstallErr(self):
        messageBox = QMessageBox(QMessageBox.Warning, "text", "text")
        messageBox.setWindowIcon(QIcon(':/icon/images/headerLogo1.png'))
        messageBox.setStandardButtons(QMessageBox.Ok)
        messageBox.setWindowTitle("Kaldırma Hatası")
        messageBox.setText("Kaldırma Başarısız !!!")
        messageBox.exec()
        self.noneBlock()
        return

    def _uninstallOk(self):
        messageBox = QMessageBox(QMessageBox.Information, "text", "text")
        messageBox.setWindowIcon(QIcon(':/icon/images/headerLogo1.png'))
        messageBox.setStandardButtons(QMessageBox.Ok)
        messageBox.setWindowTitle("Kaldırma Tamamlandı")
        messageBox.setText("Paket Başarılı Bİr Şekilde Kaldırıldı !!!")
        messageBox.exec()
        self.noneBlock()
        return

    def _updateErr(self):
        messageBox = QMessageBox(QMessageBox.Warning, "text", "text")
        messageBox.setWindowIcon(QIcon(':/icon/images/headerLogo1.png'))
        messageBox.setStandardButtons(QMessageBox.Ok)
        messageBox.setWindowTitle("Güncelleme Hatası")
        messageBox.setText("Güncelleme Başarısız !!!")
        messageBox.exec()
        self.noneBlock()
        return

    def _updateOk(self):
        messageBox = QMessageBox(QMessageBox.Information, "text", "text")
        messageBox.setWindowIcon(QIcon(':/icon/images/headerLogo1.png'))
        messageBox.setStandardButtons(QMessageBox.Ok)
        messageBox.setWindowTitle("Güncelleme Tamamlandı")
        messageBox.setText("Güncelleme Başarılı !!!")
        messageBox.exec()
        self.noneBlock()
        return

    def center(self):
        screen = QDesktopWidget().screenGeometry()
        size = self.geometry()
        self.move((screen.width() - size.width()) // 2, (screen.height() - size.height()) // 2)


    def close(self):
        tab = self.tabWidthBox.value()
        fontSize = self.fontSizeBox.value()

        self.c.updateConfig('Tab', 'Tab', str(tab))
        self.c.updateConfig('Size', 'Size', str(fontSize))

        self.done(1)



class EnterDialog(QDialog):
    def __init__(self, parent, fileName, filePath, fileDir, fileInfo, rename=True, folderPath=None):
        super().__init__(parent)
        
        self.fileName = fileName
        self.filePath = filePath
        self.fileDir = fileDir
        self.fileInfo = fileInfo
        self.rename = rename
        self.folderPath = folderPath
        
        self.initUI()
        
    def initUI(self):
        self.setModal(True)
        
        if self.rename == True:
            if self.fileDir:
                self.setWindowTitle('Rename Directory')
            else:
                self.setWindowTitle('Rename File')
        
        elif self.rename == False:
            self.setWindowTitle('Create Directory')
        
        layout = QVBoxLayout()
        
        if self.rename == True:
            self.label = QLabel(self.fileName)
            
        elif self.rename == False:
            self.label = QLabel('in: ' + self.folderPath)
        
        self.text = QLineEdit()
        self.text.setPlaceholderText("Enter name")
        
        self.buttonBox = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
        
        layout.addWidget(self.label)
        layout.addWidget(self.text)
        layout.addWidget(self.buttonBox)
        
        self.setLayout(layout)
        
        if self.rename == True:
            self.buttonBox.accepted.connect(self.accept)
        elif self.rename == False:
            self.buttonBox.accepted.connect(self.acceptMakeFolder)
        
        # for both dialogs (rename / make)
        self.buttonBox.rejected.connect(self.onReject)
        
        self.fileName = self.checkPath(self.fileName)
        self.filePath = self.checkPath(self.filePath)
        

    def checkPath(self, path):
        if '\\' in path:
            path = path.replace('\\', '/')
        return path

    def accept(self):
        if self.fileDir:

            if self.label.text() == '..':
                self.close()
    
            newName = self.text.text()
            cutPaths = self.filePath.split('/')[:-1]
            path = '/'.join(cutPaths)
            newPathName = path +  '/' + newName
            if self.text.text() == '':
                return
            try:
                os.rename(self.filePath, newPathName)
            except Exception as e:
                q = MessageBox(QMessageBox.Critical, 'Hata', 'Yol yeniden adlandırılamadı\n\n' + str(e),
                                QMessageBox.NoButton)
                q.exec_()
            
        if not self.fileDir:
            newName = self.text.text()
            oldFileName = self.filePath.split('/')[-1]
            path = self.filePath.split(oldFileName)[0]
            newFilename = path + newName
            
            # check if file exists
            for file in os.listdir(path):
                if file == newName:
                    result = MessageBox(QMessageBox.Question, 'Uyarı', 'Dosya zaten mevcut\n' +
                                                 'Devam et ?', QMessageBox.Yes | QMessageBox.No)

                    if (result.exec_() == QMessageBox.No):
                        return
            
            try:
                os.rename(self.filePath, newFilename)
            except Exception as e:
                q = MessageBox(QMessageBox.Critical, 'Hata', 'Dosya yeniden adlandırılamadı\n\n' + str(e),
                                QMessageBox.NoButton)

                q.exec_()
           
        self.close() 
    
    def acceptMakeFolder(self):
        sourcePath = self.folderPath
        newPath = self.text.text()
        if newPath.startswith('/'):
            newPath = newPath.strip('/')
        fullNewPath = sourcePath + newPath
        try:
            os.mkdir(fullNewPath)
        except Exception as e:
            q = MessageBox(QMessageBox.Critical, 'Hata', 'Dizin oluşturulamadı\n\n' + str(e),
                                QMessageBox.NoButton)

            q.exec_()
        
        self.close()

    def onReject(self):
        self.close()


class FindDeadCodeDialog(QDialog):

    def __init__(self, parent, textPad, codeView):
        super().__init__()

        self.mainWindow = parent
        self.textPad = textPad
        self.codeView = codeView

        palette = QPalette()
        role = QPalette.Background
        # palette.setColor(role, QColor('#2c2c2c'))
        self.setGeometry(self.codeView.x() + 300, self.codeView.y() + 400, 400, 300)
        self.setPalette(palette)

        self.setWindowTitle('Ölü Kod Listesi')
        self.initUI()

    def initUI(self):
        filename = os.path.basename(self.textPad.filename)
        vbox = QVBoxLayout()

        self.label = WhiteLabel(filename + ' :\n\n')
        self.label.setAlignment(Qt.AlignCenter)

        self.listWidget = ListWidget()

        updateButton = PushButton('Kaydet + Güncelle')
        updateButton.clicked.connect(self.update)
        okButton = PushButton('Tamam')
        okButton.clicked.connect(self.onClose)

        hbox = QHBoxLayout()
        # hbox.addWidget(updateButton)
        hbox.addWidget(okButton)

        vbox.addWidget(self.label)
        vbox.addWidget(self.listWidget)
        vbox.addLayout(hbox)

        self.setLayout(vbox)

        self.fillList()

        # signals
        self.listWidget.itemDoubleClicked.connect(self.gotoPos)

        self.show()

    def update(self):
        self.listWidget.clear()
        self.mainWindow.save()
        self.fillList()

    def fillList(self):
        filename = os.path.basename(self.textPad.filename)
        self.label.setText(filename)

        text = self.textPad.text()
        deadCodeChecker = DeadCodeChecker(text)
        outputList = deadCodeChecker.getList()

        self.lineNumberList = []
        self.codeList = []

        for elem in outputList:
            textList = elem.split(' ')
            self.lineNumberList.append(textList[0])
            codeText = ''

            for elem in textList[1:]:
                codeText += ' ' + elem

            self.codeList.append(codeText)

        i = 0
        for elem in self.codeList[0:-1]:
            item = QListWidgetItem()
            text = 'Satır: ' + str(self.lineNumberList[i]) + '\t-> ' + str(self.codeList[i])
            # pos = int(self.lineNumberList[i])
            item.setText(text)
            self.listWidget.addItem(item)
            i += 1

    def gotoPos(self):
        row = self.listWidget.currentRow()
        linenumber = int(str(self.lineNumberList[row]).replace(".",""))-1

        if linenumber >= 0:
            lineText = self.textPad.text(linenumber)
            rawcode = self.codeList[row]
            code = rawcode[rawcode.find("'") + 1:rawcode.rfind("'")]

            x = self.textPad.findFirst(code, False, True, False, True, True,
                                       line=linenumber, index=0)

            self.listWidget.clearSelection()
            self.textPad.setFocus()

    def onClose(self):
        self.destroy()
        self.close()  # hata ayıklayıcı ekranı çıktığında ekran kapatılsada program askıda kalıyordu, close fonk eklendi.


class PyCodeCheckerDialog(QDialog):
    
    def __init__(self, parent, textPad, codeView):
        super().__init__()

        self.mainWindow = parent
        self.textPad = textPad
        self.codeView = codeView
        
        palette = QPalette()
        role = QPalette.Background
        # palette.setColor(role, QColor('#2c2c2c'))
        self.setGeometry(self.codeView.x()+ 200, self.codeView.y(), 400, 300)
        self.setPalette(palette)

        self.setWindowTitle('Kod Stili Kontrolü Yap (pyton yazım denetleyicisi kullanarak)')
        self.initUI()

    
    def initUI(self):
        filename = os.path.basename(self.textPad.filename)
        vbox = QVBoxLayout()
        
        self.label = WhiteLabel(filename + ' :\n\n')
        self.label.setAlignment(Qt.AlignCenter)

        self.listWidget = ListWidget()
        
        updateButton = PushButton('Kaydet + Güncelle')
        updateButton.clicked.connect(self.update)
        okButton = PushButton('Tamam')
        okButton.clicked.connect(self.onClose)
        
        hbox = QHBoxLayout()
        hbox.addWidget(updateButton)
        hbox.addWidget(okButton)
        
        vbox.addWidget(self.label)
        vbox.addWidget(self.listWidget)
        vbox.addLayout(hbox)
        
        self.setLayout(vbox)
        
        self.fillList()
        
        # signals
        self.listWidget.itemDoubleClicked.connect(self.gotoPos)
        
        self.show()

    def update(self):
        self.listWidget.clear()
        self.mainWindow.save()
        self.fillList()

    def fillList(self):
        filename = os.path.basename(self.textPad.filename)
        self.label.setText(filename)
        
        check = PyCodeChecker(filename)
        text = check.getString()
        self.lineList, self.cursorList, self.textList = check.getListFromString(text)

        i = 0
        for elem in self.textList:
            item = QListWidgetItem()
            try:
                text = 'Satır: ' + str(self.lineList[i]) + '  Pozisyon: ' + str(self.cursorList[i]) + \
                        '  ' + str(self.textList[i])
                item.setText(text)
                self.listWidget.addItem(item)

            except Exception as e:
                item.destroy()

            i += 1
    
    def gotoPos(self):
        row = self.listWidget.currentRow()
        
        line = self.lineList[row]
        cursor = self.cursorList[row]
        
        self.textPad.setSelection(int(line)-1, int(cursor)-1, int(line)-1, int(cursor))

        
    def onClose(self):
        self.destroy()
        


class HelpDialog(QDialog):
    
    def __init__(self, parent):
        super().__init__()

        self.mainWindow = parent
        self.mainWindow.setObjectName("dialogMainWidget")

        self.setGeometry(self.mainWindow.x()+ 200, self.mainWindow.y(), 600, 600)

        self.setWindowTitle('Help')
        self.initUI()

    
    def initUI(self):
        self.tabs = TabWidget()
        
        tab1 = TextEdit()
        tab1.setReadOnly(True)
        tab2 = TextEdit()
        tab2.setReadOnly(True)
        tab3 = TextEdit()
        tab3.setReadOnly(True)

        layout = QVBoxLayout()
        layout.addWidget(self.tabs)

        self.tabs.addTab(tab1, "Kısayollar")
        self.tabs.addTab(tab2, "Kod Editör")
        self.tabs.addTab(tab3, "Hakkında")
        
        self.setLayout(layout)
        
#################################
        ##
        # Tab Shortcut Help
        ##
  
        shortcut_text = '''
        Shortcuts:
        
        Yeni Dosya\t-\tCtrl + N
        Dosya Aç\t-\tCtrl + O
        Dosya Kaydet\t-\tCtrl + S
        Farklı Kaydet\t\t-\tCtrl + Shift + S
        Yazdır\t\t-\tCtrl + P
        Çıkış\t\t-\tAlt + F4
        Geri Al\t\t-\tCtrl + Z
        İleri Al\t\t-\tCtrl + Shift + Z
        Kopyala\t\t-\tCtrl + C
        Kes\t\t-\tCtrl + X
        Yapıştır\t\t-\tCtrl + V
        Tümünü Seç\t-\tCtrl + A

        Otomatik Tamamlama\t-\tTab
        Ara\t\t-\tCtrl + F

        Ayarlar\t-\tF9
        Yorumlayıcı\t-\tF10
        Terminal\t-\tF11
        Çalıştır\t-\tF12
        Yakınlaştır\t-\tCtrl + +
        Uzaklaştır\t-\tCtrl + -

        Yardım\t-\tF1
        
        '''
        tab1.setText(shortcut_text)

        ##
        # Tab Codeeditor Help
        ##
        
        # codeeditor_text = '''
        # CrossCobra - The cross platform Python IDE
        #
        # CrossCobra is an Editor and Integrated Development Environment (IDE)
        # for the "Python Programming Language".
        #
        # It shows the "python source code" colored (syntax highlighting) and
        # helps you to code with its own auto-complete function.
        #
        # It can run the codefile (requirement is, that you have installed
        # Python on your OS)
        # It can also run the "Python Interpreter" and a terminal window
        # (specific for your current OS) -> this can be modified in the
        # settings (-> which where saved in pynar.ini => it's a text file)
        #
        # On the bottom-right, you find a search button with which you can
        # search the current file for a specific word.
        #
        # On the left side it has its own file-explorer to
        # copy, delete, rename ... files and folders. Use the pop-up menu
        # (right mousebutton).
        #
        # The Codeview Window on the left bottom side shows you all classes
        # and functions for the current opened file.
        #
        # You can use the right mousebutton to find dead code (using vulture)
        # or to check if your code is styled correctly (using pycodestyle)
        #
        # To use these two functions you should have installed vulture
        # and pycodestyle.
        # Use "pip3 install vulture" and "pip3 install pycodestyle" to
        # install these packages
        #
        # Crosscobra is using
        #
        # PyQt5\t\t5.11.3
        # QScintilla\t2.10.8
        # vulture\t\t1.0
        # pycodestyle\t2.4.0
        # '''
        #
        # tab2.setText(codeeditor_text)
        
        ##
        # Tab About Help
        ##

        # about_text = '''
        # CrossCobra 1.0
        #
        #
        # Programmed 2018 by morbidMo
        #
        #
        # I's open source software. You can modify and use the sourcecode -
        # make it better or fork it on github.com:
        #
        #
        # https://github.com/morten1982/crosscobra
        #
        #
        # no guarantee of using it - it's your own risk :)
        #
        #
        # CrossCobra was completely coded in Python using
        # PyQt5, QScintilla, pycodestyle, vulture
        #
        #
        # '''
        #
        # tab3.setText(about_text)
##################################
        
        self.show()

    def onClose(self):
        self.destroy()


class Main(QMainWindow):
    
    def __init__(self):
        super().__init__()
        self.initUI()
    
    def initUI(self):
        dialog = SettingsDialog(self)
        dialog.exec_()
        self.show()

if __name__ == '__main__':
    app = QApplication(sys.argv)
    main = Main()
    sys.exit(app.exec_())
